@@ -1,6 +1,6 @@
 # A Bottle of Wiki — personal wiki
 # SPDX-License-Identifier: AGPL-3.0-or-later
-# Copyright (C) 2020  Benoît Monin <benoit.monin@gmx.fr>
+# Copyright (C) 2019-2021  Benoît Monin <benoit.monin@gmx.fr>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -20,6 +20,7 @@ import errno
 import json
 import locale
 import time
+import uuid
 
 from . import alerts
 from . import config
@@ -46,6 +47,21 @@ def _build_path(pagename='', ext=''):
     return os.path.join(config.get('abow.backend_path'), pagename + ext)
 
 
+def safe_write(filepath, content):
+    '''Save some text content to a file safely
+
+    Raises OSError if the file cannot be written
+    '''
+    tmppath = filepath + uuid.uuid4().hex
+
+    with open(tmppath, 'w', encoding='utf8') as f:
+        f.write(content)
+        f.flush()
+        os.fsync(f.fileno())
+
+    os.replace(tmppath, filepath)
+
+
 def get_content(pagename):
     '''Return the content of a page or an empty string if the page is missing
     '''
@@ -70,8 +86,7 @@ def set_content(pagename, content):
         delete(pagename)
     else:
         try:
-            with open(fp, 'w', encoding='utf8') as f:
-                f.write(content.replace('\r', ''))
+            safe_write(fp, content.replace('\r', ''))
         except OSError as err:
             alerts.add('danger',
                        __('Failed to save the content of page {}: {}.').format(
@@ -138,8 +153,7 @@ def set_attr(pagename, attr, value):
 
     fp = _build_path(pagename, _ATTR_EXT)
     try:
-        with open(fp, 'w', encoding='utf8') as f:
-            json.dump(_pages_attrs[pagename], f)
+        safe_write(fp, json.dumps(_pages_attrs[pagename]))
     except OSError as err:
         alerts.add('danger',
                    __('Failed to save the attributes of page {}: {}.').format(
